<?php
namespace WebDB92\Sample;

use GuzzleHttp\Client;
use GuzzleHttp\HandlerStack;

/**
 * JSONを前提としたAPIクライアントの実装例です
 */
trait JsonAPIClientTrait
{
    /** @var \GuzzleHttp\Client */
    private $client;

    /** @var array */
    private $headers = [];

    /** @var \GuzzleHttp\HandlerStack */
    private $handler;

    public function setHeader(array $headers)
    {
        $this->headers = array_merge($this->headers, $headers);
    }

    /**
     * JSONリクエストを受け取る前提のHTTPリクエスト処理の実装例です
     */
    private function request($method, $url, array $option)
    {
        $response = $this->getClient()->request(
            $method,
            $url,
            $this->makeOption($option)
        );

        $status = $response->getStatusCode();
        if ($status === 200) {
            return json_decode($response->getBody(), true);
        }

        throw new \RuntimeException("code: $status");
    }

    /**
     * @return \GuzzleHttp\Client
     */
    private function getClient()
    {
        if (is_null($this->client)) {
            $this->client = new Client([
                'base_uri' => static::BASE_URI,
                'handler'  => $this->handler,
            ]);
        }

        return $this->client;
    }

    /**
     * @param \GuzzleHttp\HandlerStack $handler
     */
    public function setHandler(HandlerStack $handler)
    {
        $this->handler = $handler;
    }

    /**
     * Guzzleに渡すオプション配列を作成します
     */
    private function makeOption(array $option = [])
    {
        return array_merge(
            [
                'http_errors' => false,
                'headers' => $this->headers,
            ],
            $option
        );
    }
}
