<?php
namespace WebDB92\Sample;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;

/**
 * 実際にはHTTPリクエストを行わないテストです
 */
class SampleAPIClientMockTest extends APITestBase
{
    /**
     * 成功するテストです
     */
    public function test()
    {
        $expected = SampleAPIClient::USER_AGENT;
        $body = json_encode(['user-agent' => SampleAPIClient::USER_AGENT]);
        $sample_api = $this->getClient(new Response(200, [], $body));

        $this->assertEquals($expected, $sample_api->useragent());
    }

    /**
     * @expectedException RuntimeException
     * @expectedExceptionMessage code: 451
     */
    public function test_UnavailableForLegalReasons()
    {
        $sample_api = $this->getClient(new Response(451, [], 'このリクエストは検閲されました'));

        $actual = $sample_api->useragent();
        $this->assertEquals([], $actual);
    }

    /**
     * @expectedException GuzzleHttp\Exception\ConnectException
     * @expectedExceptionMessage connection error
     */
    public function test_raise_ConnectionException()
    {
        $sample_api = $this->getClient(new ConnectException("connection error", new Request('GET', '/useragent')));

        $sample_api->useragent();
    }

    /**
     * @return \WebDB92\Sample\SampleAPIClient
     */
    protected function getClient($return_value = null)
    {
        $sample_api = new SampleAPIClient;
        $mock = new MockHandler([$return_value]);
        $sample_api->setHandler(HandlerStack::create($mock));

        return $sample_api;
    }
}
